@@ -20,17 +20,18 @@ from astrocats.catalog.model import MODEL
 from astrocats.catalog.photometry import PHOTOMETRY
 from astrocats.catalog.quantity import QUANTITY
 from astrocats.catalog.realization import REALIZATION
+from astrocats.catalog.source import SOURCE
 from astrocats.catalog.utils import is_number
 from emcee.autocorr import AutocorrError
-from schwimmbad import MPIPool, SerialPool
-from six import string_types
-
+from mosfit.converter import Converter
 from mosfit.mossampler import MOSSampler
 from mosfit.printer import Printer
-from mosfit.utils import (calculate_WAIC, entabbed_json_dump,
+from mosfit.utils import (all_to_list, calculate_WAIC, entabbed_json_dump,
                           entabbed_json_dumps, flux_density_unit,
                           frequency_unit, get_model_hash, get_url_file_handle,
-                          listify, open_atomic, pretty_num, speak)
+                          listify, open_atomic, pretty_num, slugify, speak)
+from schwimmbad import MPIPool, SerialPool
+from six import string_types
 
 from .model import Model
 
@@ -66,6 +67,7 @@ class Fitter(object):
 
     _MAX_ACORC = 5
     _REPLACE_AGE = 20
+    _DEFAULT_SOURCE = {SOURCE.NAME: 'MOSFiT Paper'}
 
     def __init__(self):
         """Initialize `Fitter`."""
@@ -119,6 +121,7 @@ class Fitter(object):
                    walker_paths=[],
                    catalogs=[],
                    open_in_browser=False,
+                   limiting_magnitude=None,
                    **kwargs):
         """Fit a list of events with a list of models."""
         global model
@@ -129,6 +132,7 @@ class Fitter(object):
         self._maximum_memory = maximum_memory
         self._debug = False
         self._speak = speak
+        self._limiting_magnitude = limiting_magnitude
 
         dir_path = os.path.dirname(os.path.realpath(__file__))
         self._test = test
@@ -172,19 +176,11 @@ class Fitter(object):
 
         data = {}
 
-        new_event_list = []
-        for event in event_list:
-            if '.' in event and not event.endswith('.json'):
-                with open(event, 'r') as f:
-                    new_events = [
-                        it for s in [
-                            [y.strip("\" ") for y in x.replace(
-                                '\t', ',').split(',')]
-                            for x in f.read().splitlines()] for it in s]
-                new_event_list.extend(new_events)
-            else:
-                new_event_list.append(event)
-        event_list = new_event_list
+        # If the input is not a JSON file, assume it is either a list of
+        # transients or that it is the data from a single transient in tabular
+        # form. Try to guess the format first, and if that fails ask the user.
+        self._converter = Converter(prt, require_source=upload)
+        event_list = self._converter.generate_event_list(event_list)
 
         event_list = [x.replace('‑', '-') for x in event_list]
 
@@ -533,6 +529,10 @@ class Fitter(object):
                   walker_data=[]):
         """Load the data for the specified event."""
         prt = self._printer
+
+        if pool.is_master():
+            prt.message('loading_data', inline=True)
+
         self._walker_data = walker_data
         fixed_parameters = []
         for task in self._model._call_stack:
@@ -581,6 +581,9 @@ class Fitter(object):
 
         self._model.exchange_requests()
 
+        if pool.is_master():
+            prt.message('finding_bands', inline=True)
+
         # Run through once to set all inits.
         for root in ['output', 'objective']:
             outputs = self._model.run_stack(
@@ -701,7 +704,7 @@ class Fitter(object):
         model = self._model
         prt = self._printer
 
-        upload_this = upload and iterations > 0
+        upload_model = upload and iterations > 0
 
         if upload:
             try:
@@ -775,7 +778,7 @@ class Fitter(object):
                     p0[i].append(p)
                     dwscores.append(score)
                 else:
-                    nmap = nwalkers - len(p0[i])
+                    nmap = min(nwalkers - len(p0[i]), max(pool.size, 10))
                     dws = pool.map(draw_walker, [test_walker] * nmap)
                     p0[i].extend([x[0] for x in dws])
                     dwscores.extend([x[1] for x in dws])
@@ -987,10 +990,14 @@ class Fitter(object):
 
                     scores = [np.array(x) for x in lnprob]
                     if emim1 % kmat_chunk == 0:
-                        kmat = model.run_stack(
+                        sout = model.run_stack(
                             p[np.unravel_index(
                                 np.argmax(lnprob), lnprob.shape)],
-                            root='objective')['kmat']
+                            root='objective')
+                        kmat = sout.get('kmat', None)
+                        kdiag = sout.get('kdiagonal', None)
+                        if kdiag is not None and kmat is not None:
+                            kmat[np.diag_indices_from(kmat)] += kdiag
                     prt.status(
                         desc='fracking' if frack_now else
                         ('burning' if emi < self._burn_in else 'walking'),
@@ -1097,18 +1104,19 @@ class Fitter(object):
                 ici = ici + 1
 
         except (KeyboardInterrupt, SystemExit):
-            prt.message('ctrl_c', error=True, prefix=False)
+            prt.message('ctrl_c', error=True, prefix=False, color='!r')
             s_exception = sys.exc_info()
         except Exception:
             raise
 
         if s_exception:
             pool.close()
-            if (not prt.prompt('mc_interrupted', self._wrap_length)):
+            if (not prt.prompt('mc_interrupted')):
                 sys.exit()
 
+        prt.message('constructing')
+
         if write:
-            prt.message('saving_output')
             if self._speak:
                 speak(prt._strings['saving_output'], self._speak)
 
@@ -1132,7 +1140,6 @@ class Fitter(object):
 
         if upload:
             uentry = Entry(name=self._event_name)
-            usource = uentry.add_source(name='MOSFiT paper')
             data_keys = set()
             for task in model._call_stack:
                 if model._call_stack[task]['kind'] == 'data':
@@ -1140,7 +1147,22 @@ class Fitter(object):
                         list(model._call_stack[task].get('keys', {}).keys()))
             entryhash = entry.get_hash(keys=list(sorted(list(data_keys))))
 
-        source = entry.add_source(name='MOSFiT paper')
+        # Accumulate all the sources and add them to each entry.
+        sources = []
+        if len(self._model._references):
+            for ref in self._model._references:
+                sources.append(entry.add_source(**ref))
+        sources.append(entry.add_source(**self._DEFAULT_SOURCE))
+        source = ','.join(sources)
+
+        if upload:
+            usources = []
+            if len(self._model._references):
+                for ref in self._model._references:
+                    usources.append(uentry.add_source(**ref))
+            usources.append(uentry.add_source(**self._DEFAULT_SOURCE))
+            usource = ','.join(usources)
+
         model_setup = OrderedDict()
         for ti, task in enumerate(model._call_stack):
             task_copy = deepcopy(model._call_stack[task])
@@ -1153,6 +1175,7 @@ class Fitter(object):
              (MODEL.CODE, 'MOSFiT'), (MODEL.DATE, time.strftime("%Y/%m/%d")),
              (MODEL.VERSION, __version__), (MODEL.SOURCE, source)])
 
+        WAIC = None
         if iterations > 0:
             WAIC = calculate_WAIC(scores)
             modeldict[MODEL.SCORE] = {
@@ -1186,9 +1209,12 @@ class Fitter(object):
                 umodeldict, ignore_keys=[MODEL.DATE, MODEL.SOURCE])
             umodelnum = uentry.add_model(**umodeldict)
             if check_upload_quality:
-                if WAIC < 0.0:
-                    prt.message('no_ul_waic', [pretty_num(WAIC)])
-                    upload_this = False
+                if WAIC is None:
+                    upload_model = False
+                elif WAIC is not None and WAIC < 0.0:
+                    prt.message('no_ul_waic', ['' if WAIC is None
+                                               else pretty_num(WAIC)])
+                    upload_model = False
 
         modelnum = entry.add_model(**modeldict)
 
@@ -1219,13 +1245,16 @@ class Fitter(object):
         for xi, x in enumerate(pout):
             for yi, y in enumerate(pout[xi]):
                 # Only produce LCs for end walker state.
+                wcnt = xi * nwalkers + yi
+                if wcnt > 0:
+                    prt.message('outputting_walker', [
+                        wcnt, nwalkers * ntemps], inline=True)
                 if yi <= nwalkers:
                     output = model.run_stack(y, root='output')
                     if extra_outputs:
                         for key in extra_outputs:
                             new_val = output.get(key, [])
-                            if type(new_val).__module__ == 'numpy':
-                                new_val = new_val.tolist()
+                            new_val = all_to_list(new_val)
                             extras.setdefault(key, []).append(new_val)
                     for i in range(len(output['times'])):
                         if not np.isfinite(output['model_observations'][i]):
@@ -1241,18 +1270,60 @@ class Fitter(object):
                             photodict[PHOTOMETRY.BAND] = output['bands'][i]
                             photodict[PHOTOMETRY.MAGNITUDE] = output[
                                 'model_observations'][i]
+                            photodict[PHOTOMETRY.E_MAGNITUDE] = output[
+                                'model_variances'][i]
                         if output['observation_types'][i] == 'fluxdensity':
                             photodict[PHOTOMETRY.FREQUENCY] = output[
                                 'frequencies'][i] * frequency_unit('GHz')
                             photodict[PHOTOMETRY.FLUX_DENSITY] = output[
                                 'model_observations'][
                                     i] * flux_density_unit('µJy')
+                            photodict[
+                                PHOTOMETRY.
+                                E_LOWER_FLUX_DENSITY] = (
+                                    photodict[PHOTOMETRY.FLUX_DENSITY] - (
+                                        10.0 ** (
+                                            np.log10(photodict[
+                                                PHOTOMETRY.FLUX_DENSITY]) -
+                                            output['model_variances'][
+                                                i] / 2.5)) *
+                                    flux_density_unit('µJy'))
+                            photodict[
+                                PHOTOMETRY.
+                                E_UPPER_FLUX_DENSITY] = (10.0 ** (
+                                    np.log10(photodict[
+                                        PHOTOMETRY.FLUX_DENSITY]) +
+                                    output['model_variances'][i] / 2.5) *
+                                    flux_density_unit('µJy') -
+                                    photodict[PHOTOMETRY.FLUX_DENSITY])
                             photodict[PHOTOMETRY.U_FREQUENCY] = 'GHz'
                             photodict[PHOTOMETRY.U_FLUX_DENSITY] = 'µJy'
                         if output['observation_types'][i] == 'countrate':
                             photodict[PHOTOMETRY.COUNT_RATE] = output[
                                 'model_observations'][i]
+                            photodict[
+                                PHOTOMETRY.
+                                E_LOWER_COUNT_RATE] = (
+                                    photodict[PHOTOMETRY.COUNT_RATE] - (
+                                        10.0 ** (
+                                            np.log10(photodict[
+                                                PHOTOMETRY.COUNT_RATE]) -
+                                            output['model_variances'][
+                                                i] / 2.5)))
+                            photodict[
+                                PHOTOMETRY.
+                                E_UPPER_COUNT_RATE] = (10.0 ** (
+                                    np.log10(photodict[
+                                        PHOTOMETRY.COUNT_RATE]) +
+                                    output['model_variances'][i] / 2.5) -
+                                    photodict[PHOTOMETRY.COUNT_RATE])
                             photodict[PHOTOMETRY.U_COUNT_RATE] = 's^-1'
+                        if ('model_upper_limits' in output and
+                                output['model_upper_limits'][i]):
+                            photodict[PHOTOMETRY.UPPER_LIMIT] = bool(output[
+                                'model_upper_limits'][i])
+                        if self._limiting_magnitude is not None:
+                            photodict[PHOTOMETRY.SIMULATED] = True
                         if 'telescopes' in output and output['telescopes'][i]:
                             photodict[PHOTOMETRY.TELESCOPE] = output[
                                 'telescopes'][i]
@@ -1269,13 +1340,16 @@ class Fitter(object):
                             photodict[PHOTOMETRY.MODE] = output[
                                 'modes'][i]
                         entry.add_photometry(
-                            compare_to_existing=False, **photodict)
+                            compare_to_existing=False, check_for_dupes=False,
+                            **photodict)
 
-                        if upload_this:
+                        if upload_model:
                             uphotodict = deepcopy(photodict)
                             uphotodict[PHOTOMETRY.SOURCE] = umodelnum
                             uentry.add_photometry(
-                                compare_to_existing=False, **uphotodict)
+                                compare_to_existing=False,
+                                check_for_dupes=False,
+                                **uphotodict)
                 else:
                     output = model.run_stack(y, root='objective')
 
@@ -1310,9 +1384,10 @@ class Fitter(object):
                 realdict[REALIZATION.ALIAS] = str(ri)
                 entry[ENTRY.MODELS][0].add_realization(**realdict)
                 urealdict = deepcopy(realdict)
-                if upload_this:
+                if upload_model:
                     uentry[ENTRY.MODELS][0].add_realization(**urealdict)
                 ri = ri + 1
+        prt.message('all_walkers_written', inline=True)
 
         entry.sanitize()
         oentry = entry._ordered(entry)
@@ -1357,7 +1432,7 @@ class Fitter(object):
                     entabbed_json_dump(extras, flast, separators=(',', ':'))
                     entabbed_json_dump(extras, feven, separators=(',', ':'))
 
-        if upload_this:
+        if upload_model:
             uentry.sanitize()
             prt.message('ul_fit', [entryhash, modelhash])
             upath = '/' + '_'.join(
@@ -1377,6 +1452,40 @@ class Fitter(object):
                 else:
                     raise
 
+        if upload:
+            for ce in self._converter.get_converted():
+                dentry = Entry.init_from_file(
+                    catalog=None,
+                    name=ce[0],
+                    path=ce[1],
+                    merge=False,
+                    pop_schema=False,
+                    ignore_keys=[ENTRY.MODELS],
+                    compare_to_existing=False)
+
+                dentry.sanitize()
+                odentry = {ce[0]: uentry._ordered(dentry)}
+                dpayload = entabbed_json_dumps(odentry, separators=(',', ':'))
+                text = prt.message('ul_devent', [ce[0]], prt=False)
+                ul_devent = prt.prompt(text, kind='bool', message=False)
+                if ul_devent:
+                    dpath = '/' + slugify(
+                        ce[0] + '_' + dentry[ENTRY.SOURCES][0].get(
+                            SOURCE.BIBCODE, dentry[ENTRY.SOURCES][0].get(
+                                SOURCE.NAME, 'NOSOURCE'))) + '.json'
+                    try:
+                        dbx = dropbox.Dropbox(upload_token)
+                        dbx.files_upload(
+                            dpayload.encode(),
+                            dpath,
+                            mode=dropbox.files.WriteMode.overwrite)
+                        prt.message('ul_complete')
+                    except Exception:
+                        if self._test:
+                            pass
+                        else:
+                            raise
+
         return (entry, pout, lnprobout)
 
     def generate_dummy_data(self,
