@@ -2,7 +2,6 @@
 import importlib
 import inspect
 import json
-import logging
 import os
 from collections import OrderedDict
 from copy import deepcopy
@@ -48,6 +47,7 @@ class Model(object):
         self._print_trees = print_trees
         self._inflect = inflect.engine()
         self._inflections = {}
+        self._references = []
 
         if self._fitter:
             self._printer = self._fitter._printer
@@ -88,6 +88,7 @@ class Model(object):
                             .format(claimed_type),
                             kind='select',
                             options=type_options,
+                            message=False,
                             none_string=('None of the above, skip this '
                                          'transient.'))
 
@@ -161,7 +162,6 @@ class Model(object):
 
         with open(pp, 'r') as f:
             self._parameter_json = json.load(f, object_pairs_hook=OrderedDict)
-        self._log = logging.getLogger()
         self._modules = OrderedDict()
         self._bands = []
         self._instruments = []
@@ -191,7 +191,8 @@ class Model(object):
             else:
                 max_depth = -1
                 for tag2 in self._trees:
-                    roots.extend(self._trees[tag2]['roots'])
+                    if self.in_tree(tag, self._trees[tag2]):
+                        roots.extend(self._trees[tag2]['roots'])
                     depth = self.get_max_depth(tag, self._trees[tag2],
                                                max_depth)
                     if depth > max_depth:
@@ -231,6 +232,7 @@ class Model(object):
                 self._instruments = self._modules[task].instruments()
                 self._bands = self._modules[task].bands()
             self._modules[task].set_attributes(cur_task)
+
             # This is currently not functional for MPI
             # cur_task = self._call_stack[task]
             # mod_name = cur_task.get('class', task)
@@ -318,11 +320,11 @@ class Model(object):
                     variance_bands.append([awav, band])
                 if needs_general_variance:
                     new_call_stack[task] = deepcopy(cur_task)
-                self._printer.prt(
-                    'Anchoring variances for the following filters '
-                    '(interpolating variances for the rest): ' +
-                    (', '.join([x[1] for x in variance_bands])),
-                    wrapped=True)
+                if self._pool.is_master():
+                    self._printer.message(
+                        'anchoring_variances',
+                        [', '.join([x[1] for x in variance_bands])],
+                        wrapped=True)
                 self._modules[ptask].set_variance_bands(variance_bands)
             else:
                 new_call_stack[task] = deepcopy(cur_task)
@@ -440,7 +442,7 @@ class Model(object):
                 'input loop in `{}`.'.format(leaf))
         for tag in d:
             entry = deepcopy(d[tag])
-            new_roots = roots
+            new_roots = list(roots)
             if entry['kind'] in kinds or tag == name:
                 entry['depth'] = depth
                 if entry['kind'] in kinds:
@@ -521,6 +523,16 @@ class Model(object):
                     max_depth = new_max
         return max_depth
 
+    def in_tree(self, tag, parent):
+        """Return the maximum depth a given task is found in a tree."""
+        for child in parent.get('children', []):
+            if child == tag:
+                return True
+            else:
+                if self.in_tree(tag, parent['children'][child]):
+                    return True
+        return False
+
     def pool(self):
         """Return processing pool."""
         return self._pool
@@ -602,11 +614,20 @@ class Model(object):
                     "Failed to execute module `{}`\'s process().".format(task),
                     wrapped=True)
                 raise
+
             outputs.update(new_outs)
+
+            # Append module references
+            self._references.extend(self._modules[task]._REFERENCES)
+
             if '_delete_keys' in outputs:
                 for key in outputs['_delete_keys']:
                     del(outputs[key])
                 del(outputs['_delete_keys'])
 
             if cur_task['kind'] == root:
+                # Make sure references are unique.
+                self._references = list(map(
+                    dict, set(tuple(sorted(d.items()))
+                              for d in self._references)))
                 return outputs
